@@ -16,7 +16,11 @@ Example usage:
 u'Cabin Fever'
 """
 __author__ = "dbr/Ben"
-__version__ = "0.4"
+__version__ = "0.5"
+
+from tvdb_ui import BaseUI, ConsoleUI
+from tvdb_exceptions import (tvdb_error, tvdb_userabort, tvdb_shownotfound,
+    tvdb_seasonnotfound, tvdb_episodenotfound, tvdb_attributenotfound)
 
 class Cache:
     """Simple caching URL opener. Acts like:
@@ -34,9 +38,9 @@ class Cache:
     import tempfile
     import urllib
     try:
-        import sha1 as hasher
+        from hashlib import sha1 as hasher
     except ImportError:
-        import md5 as hasher
+        from sha import sha as hasher
 
     def __init__(self, max_age=21600, prefix="tvdb_api"):
         self.prefix = prefix
@@ -52,7 +56,7 @@ class Cache:
     def getCachePath(self, url):
         """Calculates the cache path (/temp_directory/hash_of_URL)
         """
-        cache_name = self.hasher.new(url).hexdigest()
+        cache_name = self.hasher(url).hexdigest()
         cache_path = self.os.path.join(self.tmp, cache_name)
         return cache_path
     #end getUrl
@@ -94,11 +98,11 @@ class Cache:
     #end loadUrl
 #end Cache
 
-def is_int(x):
+def can_int(x):
     """Takes a string, checks if it is numeric.
-    >>> is_int("2")
+    >>> can_int("2")
     True
-    >>> is_int("A test")
+    >>> can_int("A test")
     False
     """
     try:
@@ -110,74 +114,41 @@ def is_int(x):
 
 
 # Custom exceptions
-class tvdb_error(Exception):
-    """An error with www.thetvdb.com (Cannot connect, for example)
-    """
-    pass
-class tvdb_userabort(Exception):
-    """User aborted the interactive selection (via
-    the q command, ^c etc)
-    """
-    pass
-class tvdb_shownotfound(Exception):
-    """Show cannot be found on www.thetvdb.com (non-existant show)
-    """
-    pass
-class tvdb_seasonnotfound(Exception):
-    """Season cannot be found on www.thetvdb.com
-    """
-    pass
-class tvdb_episodenotfound(Exception):
-    """Episode cannot be found on www.thetvdb.com
-    """
-    pass
-class tvdb_attributenotfound(Exception):
-    """Raised if an episode does not have the requested
-    attribute (such as a episode name)
-    """
-    pass
 
 class ShowContainer(dict):
     """Simple dict that holds a series of Show instancies
     """
     pass
 
-class Show:
+class Show(dict):
     """Holds a dict of seasons, and show data.
     """
     def __init__(self):
-        self.seasons = {}
         self.data = {}
-    def __iter__(self):
-        for cd in self.seasons.values():
-            yield cd
+    
     def __repr__(self):
         return "<Show %s (containing %s seasons)>" % (
             self.data.get(u'seriesname', 'instance'),
-            len(self.seasons)
+            len(self)
         )
-    def has_key(self, key):
-        return dict.has_key(self.seasons, key)
-    def __setitem__(self, season_number, value):
-        dict.__setitem__(self.seasons, season_number, value)
     def __getitem__(self, key):
-        if not dict.has_key(self.seasons, key):
-            # Season number doesn't exist
-            if dict.has_key(self.data, key):
-                # check if it's a bit of data
-                return  dict.__getitem__(self.data, key)
-            else:
-                # Nope, it doesn't exist
-                # If it's numeric, it's a season number, raise season not found
-                if is_int(key):
-                    raise tvdb_seasonnotfound("Could not find season %s" % (key))
-                else:
-                    # If it's not numeric, it must be an attribute name, which
-                    # doesn't exist, so attribute error.
-                    raise tvdb_attributenotfound("Cannot find attribute %s" % (key))
-                
+        if key in self:
+            # Key is an episode, return it
+            return dict.__getitem__(self, key)
+        
+        if key in self.data:
+            # Non-numeric request is for show-data
+            return dict.__getitem__(self.data, key)
+
+        # Data wasn't found, raise appropriate error
+        if can_int(key):
+            # Episode number x was not found
+            raise tvdb_seasonnotfound("Could not find season %s" % (key))
         else:
-            return dict.__getitem__(self.seasons, key)
+            # If it's not numeric, it must be an attribute name, which
+            # doesn't exist, so attribute error.
+            raise tvdb_attributenotfound("Cannot find attribute %s" % (key))
+
     def search(self, contents = None, key = None):
         """
         Search all episodes. Can search all values, or a specific one.
@@ -195,13 +166,13 @@ class Show:
         with a bit of data containg "my first day":
 
         >>> t['Scrubs'].search("my first day") #doctest: +ELLIPSIS
-        [<__main__.Episode instance at 0x...>]
+        [<Episode 01x01 - My First Day>]
         >>>
 
         Search for "My Name Is Earl" named "Faked His Own Death":
 
         >>> t['My Name Is Earl'].search('Faked His Own Death', key = 'name') #doctest: +ELLIPSIS
-        [<__main__.Episode instance at 0x...>]
+        [<Episode 01x04 - Faked His Own Death>]
         >>>
 
         Using search results
@@ -216,10 +187,10 @@ class Show:
         >>>
         """
         if key == contents == None:
-            raise TypeError("must supply atleast one type of search")
+            raise TypeError("must supply at least one type of search")
 
         results = []
-        for cur_season in self.seasons.values():
+        for cur_season in self.values():
             for cur_ep in cur_season.values():
                 for cur_key, cur_value in cur_ep.items():
                     cur_key, cur_value = unicode(cur_key).lower(), unicode(cur_value).lower()
@@ -243,7 +214,7 @@ class Season(dict):
             len(self.keys())
         )
     def __getitem__(self, episode_number):
-        if not dict.has_key(self, episode_number):
+        if episode_number not in self:
             raise tvdb_episodenotfound
         else:
             return dict.__getitem__(self, episode_number)
@@ -270,7 +241,21 @@ class Tvdb:
     import urllib
     from BeautifulSoup import BeautifulStoneSoup
 
-    def __init__(self, interactive=False, debug=False, cache = True):
+    def __init__(self, interactive=False, debug=False, cache = True, banners = False):
+        """interactive = True uses built-in console UI is used to select
+        the correct show. When False, the first search result is used.
+        
+        debug = True shows verbose debugging information
+        
+        cache = True cacheds retrived XML are persisted to to disc
+        (under the TEMP_DIR/tvdb_api), they are
+        
+        banners = True retrives the banners for a show. These are accessed 
+        via the _banners key of a Show(), for example:
+        
+        >>> Tvdb(banners=True)['scrubs']['_banners'].keys()
+        [u'fanart', u'poster', u'series', u'season']
+        """
         self.shows = ShowContainer() # Holds all Show classes
         self.corrections = {} # Holds show-name to show_id mapping
 
@@ -283,6 +268,8 @@ class Tvdb:
         self.config['interactive'] = interactive # prompt for correct series?
         
         self.config['cache_enabled'] = cache
+        
+        self.config['banners_enabled'] = banners
 
         if self.config['cache_enabled']:
             self.cache = Cache(prefix="tvdb_api") # Caches retreived URLs in tmp dir
@@ -355,17 +342,17 @@ class Tvdb:
         calls __getitem__ on tvdb[1], there is no way to check if
         tvdb.__dict__ should have a key "1" before we auto-create it
         """
-        if not self.shows.has_key(sid):
+        if sid not in self.shows:
             self.shows[sid] = Show()
-        if not self.shows[sid].has_key(seas):
+        if seas not in self.shows[sid]: 
             self.shows[sid][seas] = Season()
-        if not self.shows[sid][seas].has_key(ep):
+        if ep not in self.shows[sid][seas]:
             self.shows[sid][seas][ep] = Episode()
         self.shows[sid][seas][ep][attrib] = value
     #end _set_item
 
     def _setShowData(self, sid, key, value):
-        if not self.shows.has_key(sid):
+        if sid not in self.shows:
             self.shows[sid] = Show()
         self.shows[sid].data.__setitem__(key, value)
 
@@ -399,47 +386,14 @@ class Tvdb:
         if len(allSeries) == 0:
             self.log.debug('Series result returned zero')
             raise tvdb_shownotfound("Show-name search returned zero results (cannot find show on TVDB)")
-
+        
         if not self.config['interactive']:
-            self.log.debug('Auto-selecting first search result')
-            return allSeries[0]
+            self.log.debug('Auto-selecting first search result using BaseUI')
+            return BaseUI(self.log).selectSeries(allSeries)
         else:
-            self.log.debug('Interactivily selecting show')
-            print "TVDB Search Results:"
-            for i in range(len(allSeries[:6])): # list first 6 search results
-                i_show = i + 1 # Start at more human readable number 1 (not 0)
-                self.log.debug('Showing allSeries[%s] = %s)' % (i_show, allSeries[i]))
-                print "%s -> %s (tvdb id: %s)" % (
-                    i_show,
-                    allSeries[i]['name'].encode("UTF-8","ignore"),
-                    allSeries[i]['sid'].encode("UTF-8","ignore")
-                )
-
-            while True: # return breaks this loop
-                try:
-                    print "Enter choice (first number, ? for help):"
-                    ans = raw_input()
-                except KeyboardInterrupt:
-                    raise tvdb_userabort("User aborted (^c keyboard interupt)")
-
-                self.log.debug('Got choice of: %s' % (ans))
-                try:
-                    selected_id = int(ans) - 1 # The human entered 1 as first result, not zero
-                    self.log.debug('Trying to return ID: %d' % (selected_id))
-                    return allSeries[ selected_id ]
-                except ValueError: # Input was not number
-                    if ans == "q":
-                        self.log.debug('Got quit command (q)')
-                        raise tvdb_userabort("User aborted ('q' quit command)")
-                    elif ans == "?":
-                        print "## Help"
-                        print "# Enter the number that corresponds to the correct show."
-                        print "# ? - this help"
-                        print "# q - abort tvnamer"
-                    else:
-                        self.log.debug('Unknown keypress %s' % (ans))
-                #end try
-            #end while not valid_input
+            self.log.debug('Interactivily selecting show using ConsoleUI')
+            return ConsoleUI(self.log).selectSeries(allSeries)
+            
     #end _getSeries
 
     def _getShowData(self, sid):
@@ -465,40 +419,33 @@ class Tvdb:
         #end for series
         
         # Parse banners
-        self.log.debug('Getting season banners for %s' % (sid))
-        bannersSoup = self._getsoupsrc( self.config['url_seriesBanner'] % (sid) )    
-        banners = {}
-        for cur_banner in bannersSoup.findAll('banner'):
-            # bannerpath: append to <mirrorpath>/banners/ to get actual URL
-            # bannertype: can be series, season, poster or fanart
-            # bannertype: text, graphical or blank
-            # language: Nobody knows
-            # Season: season number this relates to (if appropriate)
-            # vignette path: like banner path, but vignetted
-            # thumbnailpath: like banner path, 300px < 20KB
-            # colors : colours selected by the artist. 1st, light accent, 2nd dark accent, 3rd is netural mid tone
-            bid = cur_banner.id.string
-            btype = cur_banner.bannertype.string
-            btype2 = cur_banner.bannertype2.string
-            if not btype in banners:
-                banners[btype] = {}
-            if not btype2 in banners[btype]:
-                banners[btype][btype2] = {}
-            if not bid in banners[btype][btype2]:
-                banners[btype][btype2][bid] = {}
+        if self.config['banners_enabled']:
+            self.log.debug('Getting season banners for %s' % (sid))
+            bannersSoup = self._getsoupsrc( self.config['url_seriesBanner'] % (sid) )    
+            banners = {}
+            for cur_banner in bannersSoup.findAll('banner'):
+                bid = cur_banner.id.string
+                btype = cur_banner.bannertype.string
+                btype2 = cur_banner.bannertype2.string
+                if not btype in banners:
+                    banners[btype] = {}
+                if not btype2 in banners[btype]:
+                    banners[btype][btype2] = {}
+                if not bid in banners[btype][btype2]:
+                    banners[btype][btype2][bid] = {}
             
-            self.log.debug("Banner: %s", bid)
-            for cur_element in cur_banner.findChildren():
-                self.log.debug("Banner info: %s = %s" % (cur_element.name, cur_element.string))
-                banners[btype][btype2][bid][cur_element.name] = cur_element.string
+                self.log.debug("Banner: %s", bid)
+                for cur_element in cur_banner.findChildren():
+                    self.log.debug("Banner info: %s = %s" % (cur_element.name, cur_element.string))
+                    banners[btype][btype2][bid][cur_element.name] = cur_element.string
             
-            for k, v in banners[btype][btype2][bid].items():
-                if k.endswith("path"):
-                    new_key = "_%s" % (k)
-                    new_url = self.config['url_bannerPath'] % (v)
-                    banners[btype][btype2][bid][new_key] = new_url
+                for k, v in banners[btype][btype2][bid].items():
+                    if k.endswith("path"):
+                        new_key = "_%s" % (k)
+                        new_url = self.config['url_bannerPath'] % (v)
+                        banners[btype][btype2][bid][new_key] = new_url
 
-        self._setShowData(sid, "_banners", banners)
+            self._setShowData(sid, "_banners", banners)
 
         # Parse episode data
         self.log.debug('Getting all episodes of %s' % (sid))
@@ -522,7 +469,7 @@ class Tvdb:
         already been grabbed), or grabs all episodes and returns
         the correct SID.
         """
-        if self.corrections.has_key(name):
+        if name in self.corrections:
             self.log.debug('Correcting %s to %s' % (name, self.corrections[name]) )
             sid = self.corrections[name]
         else:
@@ -533,7 +480,7 @@ class Tvdb:
 
             self.corrections[name] = sid
             self._getShowData( sid )
-        #end if self.corrections.has_key
+        #end if name in self.corrections
         return sid
     #end _nameToSid
 
@@ -547,11 +494,6 @@ class Tvdb:
         return self.shows[sid]
     #end __getitem__
 
-    def __setitem__(self, key, value):
-        self.log.debug('Setting %s = %s' % (key, value))
-        self.shows[key] = value
-    #end __getitem__
-
     def __str__(self):
         return str(self.shows)
     #end __str__
@@ -559,9 +501,12 @@ class Tvdb:
 
 import unittest
 class test_tvdb(unittest.TestCase):
-    # Setting up this in def Setup() is slower, and
-    # it doesn't need to be called for eachtest
-    t = Tvdb()
+    # Used to store the cached instance of Tvdb()
+    t = None
+    
+    def setUp(self):
+        if self.t is None:
+            self.__class__.t = Tvdb(cache = False, banners = False)
      
     def test_different_case(self):
         """Checks the auto-correction of show names is working.
@@ -624,10 +569,17 @@ class test_tvdb(unittest.TestCase):
         """
         self.assertEquals(
             self.t['Battlestar Galactica (2003)'][1][6]['overview'].startswith(
-                '''When a new copy of Doral, a Cylon who had been previously'''),
+                'When a new copy of Doral, a Cylon who had been previously'),
             True
         )
     
+    def test_search_no_params_error(self):
+        """Checks not supplying search info rasies TypeError"""
+        self.assertRaises(
+            TypeError,
+            lambda: self.t['Scrubs'].search()
+        )
+    
     def test_show_iter(self):
         """Iterating over a show returns each seasons
         """
@@ -649,12 +601,57 @@ class test_tvdb(unittest.TestCase):
         )
     
     def test_episode_data(self):
-        """Checking the firstaired value is retrived"""
+        """Check the firstaired value is retrived
+        """
         self.assertEquals(
             self.t['lost']['firstaired'],
             '2004-09-22'
         )
 
+    def test_repr_show(self):
+        """Check repr() of Season
+        """
+        self.assertEquals(
+            repr(self.t['CNNNN']),
+            "<Show Chaser Non-Stop News Network (CNNNN) (containing 2 seasons)>"
+        )
+    def test_repr_season(self):
+        """Check repr() of Season
+        """
+        self.assertEquals(
+            repr(self.t['CNNNN'][1]),
+            "<Season instance (containing 9 episodes)>"
+        )
+    def test_repr_episode(self):
+        """Check repr() of Episode
+        """
+        self.assertEquals(
+            repr(self.t['CNNNN'][1][1]),
+            "<Episode 01x01 - September 19, 2002 (20:30 - 21:00)>"
+        )
+    
+    def test_have_banners(self):
+        """Check banners at least one banner is found
+        """
+        self.assertEquals(
+            len(self.t['scrubs']['_banners']) > 0,
+            True
+        )
+    
+    def test_banner_url(self):
+        """Checks banner URLs start with http://
+        """
+        orig_banners_enabled = self.t.config['banners_enabled']
+        self.t.config['banners_enabled'] = True
+        for banner_type, banner_data in self.t['scrubs']['_banners'].items():
+            for res, res_data in banner_data.items():
+                for bid, banner_info in res_data.items():
+                    self.assertEquals(
+                        banner_info['_bannerpath'].startswith("http://"),
+                        True
+                    )
+        self.t.config['banners_enabled'] = orig_banners_enabled
+    
     def test_doctest(self):
         """Check docstring examples works"""
         import doctest
@@ -684,8 +681,10 @@ def main():
     from optparse import OptionParser
 
     parser = OptionParser(usage="%prog [options]")
-    parser.add_option(  "-t", "--tests", action="store_true", default=False, dest="dotests",
-                        help="Run unittests (mostly useful for development)")
+    parser.add_option(
+        "-t", "--tests", action="store_true", default=False, dest="dotests",
+        help="Run unittests (mostly useful for development)"
+    )
 
     opts, args = parser.parse_args()
 
