@@ -13,7 +13,7 @@ Uses data from www.thetvdb.com via tvdb_api
 """
 
 __author__ = "dbr/Ben"
-__version__ = "0.4"
+__version__ = "0.5"
 
 import os, sys, re
 from optparse import OptionParser
@@ -24,13 +24,27 @@ from tvdb_api import Tvdb
 
 config = {}
 
+### Start user config
+
 # The format of the renamed files (with and without episode names)
 config['with_ep_name'] = '%(seriesname)s - [%(seasno)02dx%(epno)02d] - %(epname)s.%(ext)s'
 config['without_ep_name'] = '%(seriesname)s - [%(seasno)02dx%(epno)02d].%(ext)s'
 
+# Whitelist of valid filename characters
 config['valid_filename_chars'] = """0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!@£$%^&*()_+=-[]{}"'.,<>`~? """
+
+# Force the stripping of invalid Windows characters, even if the current 
+# platform is not detected as Windows
+config['force_windows_compliant_filenames'] = False
+
+### End user config
+
 config['valid_filename_chars_regex'] = re.escape(config['valid_filename_chars'])
 
+if sys.platform == "win32" or config['force_windows_compliant_filenames']:
+    # " * : < > ? | \ are all invalid on Windows
+    config['valid_filename_chars'] = config['valid_filename_chars'].replace("\"*:<>?|\\","")
+
 # Regex's to parse filenames with. Must have 3 groups, seriesname, season number
 # and episode number. Use (?: optional) non-capturing groups if you need others.
 config['name_parse'] = [
@@ -47,10 +61,11 @@ config['name_parse'] = [
 ]
 
 
-def findFiles(args):
+def findFiles(args, recursive = False, verbose = False):
     """
     Takes a list of files/folders, grabs files inside them. Does not recurse
-    more than one level (if a folder is supplied, it will list files within)
+    more than one level (if a folder is supplied, it will list files within),
+    unless recurse is True, in which case it will recursivly find all files.
     """
     allfiles = []
     for cfile in args:
@@ -59,6 +74,13 @@ def findFiles(args):
                 newpath = os.path.join(cfile, sf)
                 if os.path.isfile(newpath):
                     allfiles.append(newpath)
+                else:
+                    if recursive:
+                        if verbose: print "Recursivly scanning %s" % (newpath)
+                        allfiles.extend(
+                            findFiles([newpath], recursive = recursive, verbose = verbose)
+                        )
+                    #end if recursive
                 #end if isfile
             #end for sf
         elif os.path.isfile(cfile):
@@ -155,6 +177,113 @@ def renameFile(oldfile, newfile, force=False):
         return True
     #end if new_exists
 
+def processFile(t, opts, cfile):
+    try:
+        # Ask for episode name from tvdb_api
+        epname = t[ cfile['file_seriesname'] ][ cfile['seasno'] ][ cfile['epno'] ]['episodename']
+    except tvdb_shownotfound:
+        # No such show found.
+        # Use the show-name from the files name, and None as the ep name
+        sys.stderr.write("! Warning: Show %s not found (in %s)\n" % (
+            cfile['file_seriesname'],
+            cfile['filepath'] )
+        )
+
+        cfile['seriesname'] = cfile['file_seriesname']
+        cfile['epname'] = None
+    except (tvdb_seasonnotfound, tvdb_episodenotfound, tvdb_attributenotfound):
+        # The season, episode or name wasn't found, but the show was.
+        # Use the corrected show-name, but no episode name.
+        sys.stderr.write("! Warning: Episode name not found for %s (in %s)\n" % (
+            cfile['file_seriesname'],
+            cfile['filepath'] )
+        )
+
+        cfile['seriesname'] = t[ cfile['file_seriesname'] ]['seriesname']
+        cfile['epname'] = None
+    except tvdb_error, errormsg:
+        # Error communicating with thetvdb.com
+        sys.stderr.write(
+            "! Warning: Error contacting www.thetvdb.com:\n%s\n" % (errormsg)
+        )
+
+        cfile['seriesname'] = t[ cfile['file_seriesname'] ]['seriesname']
+        cfile['epname'] = None
+    except tvdb_userabort, errormsg:
+        # User aborted selection (q or ^c)
+        print "\n", errormsg
+        sys.exit(1)
+    else:
+        cfile['epname'] = epname
+        cfile['seriesname'] = t[ cfile['file_seriesname'] ]['seriesname'] # get the corrected seriesname
+
+    # Format new filename, strip unwanted characters
+    newname = formatName(cfile)
+    newname = cleanName(newname)
+
+    # Append new filename (with extension) to path
+    oldfile = os.path.join(
+        cfile['filepath'],
+        cfile['filename'] + "." + cfile['ext']
+    )
+    # Join path to new file name
+    newfile = os.path.join(
+        cfile['filepath'],
+        newname
+    )
+
+    # Show new/old filename
+    print "#"*20
+    print "Old name: %s" % ( cfile['filename'] + "." + cfile['ext'] )
+    print "New name: %s" % ( newname )
+
+    # Either always rename, or prompt user
+    if opts.always or (not opts.interactive):
+        rename_result = renameFile(oldfile, newfile, force=opts.force)
+        if rename_result:
+            print "..auto-renaming"
+        else:
+            print "..not renamed"
+        #end if rename_result
+
+        return # next filename!
+    #end if always
+
+    ans = None
+    while ans not in ['y', 'n', 'a', 'q', '']:
+        print "Rename?"
+        print "([y]/n/a/q)",
+        try:
+            ans = raw_input().strip()
+        except KeyboardInterrupt, errormsg:
+            print "\n", errormsg
+            sys.exit(1)
+        #end try
+    #end while
+
+    if len(ans) == 0:
+        print "Renaming (default)"
+        rename_result = renameFile(oldfile, newfile, force=opts.force)
+    elif ans[0] == "a":
+        opts.always = True
+        rename_result = renameFile(oldfile, newfile, force=opts.force)
+    elif ans[0] == "q":
+        print "Aborting"
+        sys.exit(1)
+    elif ans[0] == "y":
+        rename_result = renameFile(oldfile, newfile, force=opts.force)
+    elif ans[0] == "n":
+        print "Skipping"
+        return
+    else:
+        print "Invalid input, skipping"
+    #end if ans
+    if rename_result:
+        print "..renamed"
+    else:
+        print "..not renamed"
+    #end if rename_result
+#end processFile
 
 def main():
     parser = OptionParser(usage="%prog [options] <file or directories>")
@@ -165,6 +294,8 @@ def main():
                         help="selects first search result, requires no human intervention once launched", default=False)
     parser.add_option(  "-i", "--interactive", action="store_true", dest="interactive", default=True,
                         help="interactivly select correct show from search results [default]")
+    parser.add_option(  "-r", "--recursive", action="store_true", dest="recursive", default=True,
+                        help="recursivly search supplied directories for files to rename")
     parser.add_option(  "-a", "--always", action="store_true", default=False, dest="always",
                         help="always renames files (but still lets user select correct show). Can be changed during runtime with the 'a' prompt-option")
     parser.add_option(  "-f", "--force", action="store_true", default=False, dest="force",
@@ -184,7 +315,7 @@ def main():
         parser.error("No filenames or directories supplied")
     #end if len(args)
 
-    allFiles = findFiles(args)
+    allFiles = findFiles(args, opts.recursive, verbose = opts.debug)
     validFiles = processNames(allFiles, verbose = opts.debug)
 
     if len(validFiles) == 0:
@@ -203,112 +334,7 @@ def main():
 
     for cfile in validFiles:
         print "# Processing %(file_seriesname)s (season: %(seasno)d, episode %(epno)d)" % (cfile)
-        try:
-            # Ask for episode name from tvdb_api
-            epname = t[ cfile['file_seriesname'] ][ cfile['seasno'] ][ cfile['epno'] ]['episodename']
-        except tvdb_shownotfound:
-            # No such show found.
-            # Use the show-name from the files name, and None as the ep name
-            sys.stderr.write("! Warning: Show %s not found (in %s)\n" % (
-                cfile['file_seriesname'],
-                cfile['filepath'] )
-            )
-
-            cfile['seriesname'] = cfile['file_seriesname']
-            cfile['epname'] = None
-        except (tvdb_seasonnotfound, tvdb_episodenotfound, tvdb_attributenotfound):
-            # The season, episode or name wasn't found, but the show was.
-            # Use the corrected show-name, but no episode name.
-            sys.stderr.write("! Warning: Episode name not found for %s (in %s)\n" % (
-                cfile['file_seriesname'],
-                cfile['filepath'] )
-            )
-
-            cfile['seriesname'] = t[ cfile['file_seriesname'] ]['seriesname']
-            cfile['epname'] = None
-        except tvdb_error, errormsg:
-            # Error communicating with thetvdb.com
-            sys.stderr.write(
-                "! Warning: Error contacting www.thetvdb.com:\n%s\n" % (errormsg)
-            )
-
-            cfile['seriesname'] = t[ cfile['file_seriesname'] ]['seriesname']
-            cfile['epname'] = None
-        except tvdb_userabort, errormsg:
-            # User aborted selection (q or ^c)
-            print "\n", errormsg
-            sys.exit(1)
-        else:
-            cfile['epname'] = epname
-            cfile['seriesname'] = t[ cfile['file_seriesname'] ]['seriesname'] # get the corrected seriesname
-
-        # Format new filename, strip unwanted characters
-        newname = formatName(cfile)
-        newname = cleanName(newname)
-
-        # Append new filename (with extension) to path
-        oldfile = os.path.join(
-            cfile['filepath'],
-            cfile['filename'] + "." + cfile['ext']
-        )
-        # Join path to new file name
-        newfile = os.path.join(
-            cfile['filepath'],
-            newname
-        )
-
-        # Show new/old filename
-        print "#"*20
-        print "Old name: %s" % ( cfile['filename'] + "." + cfile['ext'] )
-        print "New name: %s" % ( newname )
-
-        # Either always rename, or prompt user
-        if opts.always or (not opts.interactive):
-            rename_result = renameFile(oldfile, newfile, force=opts.force)
-            if rename_result:
-                print "..auto-renaming"
-            else:
-                print "..not renamed"
-            #end if rename_result
-
-            continue # next filename!
-        #end if always
-
-        ans = None
-        while ans not in ['y', 'n', 'a', 'q', '']:
-            print "Rename?"
-            print "([y]/n/a/q)",
-            try:
-                ans = raw_input().strip()
-            except KeyboardInterrupt, errormsg:
-                print "\n", errormsg
-                sys.exit(1)
-            #end try
-        #end while
-
-        if len(ans) == 0:
-            print "Renaming (default)"
-            rename_result = renameFile(oldfile, newfile, force=opts.force)
-        elif ans[0] == "a":
-            opts.always = True
-            rename_result = renameFile(oldfile, newfile, force=opts.force)
-        elif ans[0] == "q":
-            print "Aborting"
-            sys.exit(1)
-        elif ans[0] == "y":
-            rename_result = renameFile(oldfile, newfile, force=opts.force)
-        elif ans[0] == "n":
-            print "Skipping"
-            continue
-        else:
-            print "Invalid input, skipping"
-        #end if ans
-        if rename_result:
-            print "..renamed"
-        else:
-            print "..not renamed"
-        #end if rename_result
-    #end for cfile
+        processFile(t, opts, cfile)
     print "# Done"
 #end main
 
